function L = laplacian(m, h)
%  L = laplacian(m, h)
%
% Description:
%   Discrete linear Laplacian operator on a regular grid (1D and 2D)
%
% Input arguments:
%   m - dimensions of operator
%   h - grid size
%
% Output arguments:
%   L - Laplacian operator
%

if any(m < 4), error('not enough elements to generate Laplacian'), end
if nargin < 2, h = 1; end
  
dim = length(m); % dimension of Laplacian

if length(h) == 1 % equidistant

  switch dim
    case 1
      e = ones(m, 1);
      L = spdiags([e -2*e e], -1:1, m, m); 
      L = 1/h^2 * L;

    case 2
      e = ones(m(2), 1);
      L1 = spdiags([e -2*e e], -1:1, m(2), m(2)); 
               
      e = ones(m(1), 1);
      L2 = spdiags([e -2*e e], -1:1, m(1), m(1)); 
      
      L = 1/h^2 * (kron(speye(m(1)), L1) + kron(L2, speye(m(2))));

    otherwise
      error('Laplacian only implemented for 1D and 2D')
  end
else % not equidistant
end

return